@@ -7,6 +7,9 @@ setAs("matrix", "TsparseMatrix",
       else if(is.logical(from)) as(Matrix(from, sparse=TRUE), "TsparseMatrix")
       else stop("not-yet-implemented coercion to \"TsparseMatrix\""))
 
+setAs("numeric", "TsparseMatrix",
+      function(from) as(as.matrix(from), "TsparseMatrix"))
+
 setAs("TsparseMatrix", "matrix",
       function(from) .Call(dgTMatrix_to_matrix, as(from, "dgTMatrix")))
 
@@ -733,7 +736,8 @@ replTmat <- function (x, i, j, ..., value)
 .TM.repl.i.2col <- function (x, i, j, ..., value)
 {
     nA <- nargs()
-    if(nA != 3) stop("nargs() = ", nA, " should never happen; please report.")
+    if(nA != 3)
+	stop(gettextf("nargs() = %d should never happen; please report.", nA))
 
     ## else: nA == 3  i.e.,  M [ cbind(ii,jj) ] <- value or M [ Lmat ] <- value
     if(is.logical(i)) {
